/*
 * client_config.hpp
 *
 * Configuration for a client. All config information is actually received over
 * the network by the server.
 *
 * Copyright (c) Stephen Thompson, 2008.
 * Licensed for non-commercial use only. See LICENCE.txt for details.
 *
 */

#ifndef CLIENT_CONFIG_HPP
#define CLIENT_CONFIG_HPP

#include "coercri/gfx/graphic.hpp"
#include "coercri/network/network_connection.hpp"

#include "boost/scoped_ptr.hpp"
using namespace boost;

#include <vector>
using namespace std;

class Anim;
class Control;
class GfxManager;
class Overlay;
class PotionRenderer;
class SkullRenderer;

class ClientConfig {
public:
    // ctor loads the output of GameConfig::serializeAll. (Exception: Graphics are not deleted from the GfxManager.)
    ClientConfig(Coercri::NetworkConnection &remote, GfxManager &gm);

    // dtor: Deletes all the created objects.
    ~ClientConfig();
    
    // Lookup functions -- convert IDs to pointers.
    Coercri::Graphic * getGraphic(int id) const;
    Anim * getAnim(int id) const;
    Overlay * getOverlay(int id) const;
    Control * getControl(int id) const;

    // Other lookup functions
    PotionRenderer *getPotionRenderer() const { return potion_renderer.get(); }
    SkullRenderer *getSkullRenderer() const { return skull_renderer.get(); }
    const Coercri::Graphic *getMenuCentre() const { return menu_centre; }
    const Coercri::Graphic *getMenuEmpty() const { return menu_empty; }
    const Coercri::Graphic *getMenuHighlight() const { return menu_highlight; }
    const Coercri::Graphic *getWinnerImage() const { return winner_image; }
    const Coercri::Graphic *getLoserImage() const { return loser_image; }
    
private:
    vector<Coercri::Graphic*> graphics;
    vector<Anim*> anims;
    vector<Overlay*> overlays;
    vector<Control*> controls;
    scoped_ptr<PotionRenderer> potion_renderer;
    scoped_ptr<SkullRenderer> skull_renderer;
    Coercri::Graphic *menu_centre, *menu_empty, *menu_highlight;
    Coercri::Graphic *winner_image, *loser_image;
};

#endif
