/*
 * FILE:
 *   basic_socket.hpp
 *
 * PURPOSE:
 *   Simple unreliable socket (usually maps directly to a UDP socket).
 *   Binds to a certain port number.
 *   Can receive (from any address), send to a specific address, or send to
 *   all addresses (broadcast).
 *
 * AUTHOR:
 *   Stephen Thompson, 2008
 *
 * COPYRIGHT:
 *   Public Domain.
 *
 */

#ifndef COERCRI_BASIC_SOCKET_HPP
#define COERCRI_BASIC_SOCKET_HPP

#include "boost/noncopyable.hpp"
#include "boost/shared_ptr.hpp"

#include <string>

namespace Coercri {

    class BasicSocket : boost::noncopyable {
    public:
        virtual ~BasicSocket() { }

        // Receive a message. Returns false if no message is waiting. Otherwise returns true and
        // copies the message & the address of the sender into the two parameters.
        virtual bool receive(std::string &address, std::string &msg) = 0;

        // Send a message to a specific address.
        virtual void send(const std::string &address, const std::string &msg) = 0;
        
        // Broadcast a message.
        virtual void broadcast(const std::string &msg) = 0;
    };

}

#endif
