/*
 * FILE:
 *   network_driver.hpp
 *
 * PURPOSE:
 *   Simple networking interface.
 *
 * AUTHOR:
 *   Stephen Thompson, 2008
 *
 * COPYRIGHT:
 *   Public Domain.
 *   
 */

#ifndef COERCRI_NETWORK_DRIVER_HPP
#define COERCRI_NETWORK_DRIVER_HPP

#include "boost/noncopyable.hpp"
#include "boost/shared_ptr.hpp"

#include <string>
#include <vector>

namespace Coercri {

    class BasicSocket;
    class NetworkConnection;
    
    class NetworkDriver : boost::noncopyable {
    public:
        virtual ~NetworkDriver() { }
        
        // Opening outgoing connections:
        virtual boost::shared_ptr<NetworkConnection> openConnection(const std::string &host, int port) = 0;

        
        // Listening for incoming connections:

        // set the server port
        // (initially -1.)
        // (can only listen on one port at a time for now. set to -1 to disable incoming connections.)
        // (NB changing the server port will forcibly reset any existing incoming connections.)
        virtual void setServerPort(int port) = 0;

        // enable/disable the server.
        // (initially enabled.)
        // (disabling the server will disconnect all existing connections, and refuse any new incoming ones.)
        virtual void enableServer(bool enabled) = 0;
        
        // (this returns a list of new incoming connections that have appeared since we were last called.)
        typedef std::vector<boost::shared_ptr<NetworkConnection> > Connections;
        virtual Connections pollIncomingConnections() = 0;

        
        // Process any pending network events. This should be called
        // regularly from the program's main loop, to ensure that
        // packets get sent/received promptly. Returns TRUE if it "did
        // anything".
        virtual bool doEvents() = 0;


        // Create a BasicSocket on a given port.
        virtual boost::shared_ptr<BasicSocket> createBasicSocket(int port) = 0;
    };

}

#endif
