/*
 * keyboard_controller.hpp
 *
 * KeyboardController: Controller that reads input from the Keyboard
 * Keys to use must be configured in the constructor.
 *
 * Copyright (c) Stephen Thompson, 2008.
 * Licensed for non-commercial use only. See LICENCE.txt for details.
 *
 */

#ifndef KEYBOARD_CONTROLLER_HPP
#define KEYBOARD_CONTROLLER_HPP

#include "controller.hpp"

#include "coercri/gfx/raw_key.hpp"
#include "coercri/gfx/window.hpp"

#include "boost/scoped_ptr.hpp"
using namespace boost;

class KeyboardListener;

class KeyboardController : public Controller {
public:
    KeyboardController(Coercri::RawKey up_key_, Coercri::RawKey right_key_, Coercri::RawKey down_key_,
                       Coercri::RawKey left_key_, Coercri::RawKey fire_key_, Coercri::RawKey suicide_key_,
                       shared_ptr<Coercri::Window> w);
    virtual ~KeyboardController();
    
    virtual void get(MapDirection &dir, bool &centred, bool &fire_held,
                     bool &suicide_held) const;
    
    // currently we set F1--F12 to be the cheat keys.
    virtual int getCheatKey() const;
    
private:
    friend class KeyboardListener;
    scoped_ptr<KeyboardListener> kbd_listener;
    shared_ptr<Coercri::Window> win;
    Coercri::RawKey up_key, right_key, down_key, left_key, fire_key, suicide_key;
    bool up, right, down, left, fire, suicide;
    bool cheat[12];
};

#endif
