/*
 * knights_game.hpp
 *
 * Top level class for Knights. The main() function constructs this
 * class and calls runKnights().
 * 
 * Copyright (c) Stephen Thompson, 2008.
 * Licensed for non-commercial use only. See LICENCE.txt for details.
 *
 */

#ifndef KNIGHTS_GAME_HPP
#define KNIGHTS_GAME_HPP

#include "coercri/gfx/font.hpp"
#include "coercri/network/network_connection.hpp"
#include "coercri/network/network_driver.hpp"

#include "boost/shared_ptr.hpp"
using namespace boost;

#include <iosfwd>
#include <memory>
#include <string>
using namespace std;

class ClientConfig;
class Controller;
class GfxManager;
class KnightsGameImpl;
class Options;
class Screen;
class ServerConfig;
class SoundManager;

class KnightsGame {
public:
    explicit KnightsGame(const string &resource_dir);

    // Start the main loop
    void runKnights();

    // Screen change and Quit requests
    void requestScreenChange(auto_ptr<Screen> screen);
    void requestQuit();

    // Resets global game data (called when we go back to the title screen)
    void resetAll();

    // "Get" methods
    shared_ptr<Coercri::Font> getFont() const;  // the Coercri font being used for the guichan global widget font.
    const Controller & getLeftController() const;
    const Controller & getRightController() const;
    GfxManager & getGfxManager() const;
    SoundManager & getSoundManager() const;
    void getDesktopResolution(int &w, int &h) const;
    
    // Options. KnightsGame ctor will load options from disk. setAndSaveOptions will set new
    // options and save them back to disk.
    const Options & getOptions() const;
    void setAndSaveOptions(const Options &);

    // to work around a guichan bug. This should be called if new buttons are created
    void repeatLastMouseInput();
    
public:
    // The network and game state is made public, and manipulated directly by Screens:
    bool server; // True if server or local; false if client
    shared_ptr<Coercri::NetworkDriver> net_driver;
    shared_ptr<Coercri::NetworkConnection> remote;   // NULL if local
    shared_ptr<ServerConfig> server_config;
    shared_ptr<ClientConfig> client_config;

private:
    void executeScreenChange();
    void setupControllers();
    void setupGfxResizer();
    
private:
    shared_ptr<KnightsGameImpl> pimpl;
};

#endif
