/*
 * loading_screen.hpp
 *
 * Loading Screen
 * --- servers or local: loads ServerConfig
 * --- server: sends config to client
 * --- client: receives config from server
 * Sets KnightsGame::server_config or KnightsGame::client_config
 * Continues to MenuScreen.
 * 
 * Copyright (c) Stephen Thompson, 2008.
 * Licensed for non-commercial use only. See LICENCE.txt for details.
 *
 */

#ifndef LOADING_SCREEN_HPP
#define LOADING_SCREEN_HPP

#include "screen.hpp"

#ifndef DISABLE_MULTITHREADING
class GfxManager;
class SoundManager;
#include "SDL.h"
#endif

class LoadingScreen : public Screen {
public:
    LoadingScreen() : knights_game(0) { }
    virtual bool start(KnightsGame &knights_game, shared_ptr<Coercri::Window> window, gcn::Gui &gui);
    virtual ~LoadingScreen();
    virtual unsigned int getUpdateInterval() { return 50; }
    virtual void update();
    virtual void draw(Coercri::GfxContext &gc);
private:
    shared_ptr<Coercri::Window> window;
    shared_ptr<Coercri::WindowListener> window_listener;
    KnightsGame *knights_game;

#ifndef DISABLE_MULTITHREADING
public:
    // TODO: Should not really call SDL directly here. Should either
    // wrap SDL threads in a Coercri interface (as we have done for
    // all other SDL accesses in the program) or else use something
    // like Boost.Threads instead.

    struct ThreadContext {
        shared_ptr<ServerConfig> result;
        bool error;
        std::string error_message;
        SDL_mutex * mutex;
        GfxManager &gm;
        SoundManager &sm;

        ThreadContext(GfxManager &gm_, SoundManager &sm_)
            : error(false), mutex(SDL_CreateMutex()), gm(gm_), sm(sm_)
        { }
        ~ThreadContext() { SDL_DestroyMutex(mutex); }
    };

private:
    ThreadContext *my_context;
    SDL_Thread *my_thread;
#endif
};

#endif

