/*
 * menu_constraint.hpp
 *
 * Copyright (c) Stephen Thompson, 2008.
 * Licensed for non-commercial use only. See LICENCE.txt for details.
 *
 */

#ifndef MENU_CONSTRAINT_HPP
#define MENU_CONSTRAINT_HPP

#include "boost/shared_ptr.hpp"
using namespace boost;

#include <string>
using namespace std;

class MenuSelections;
class MenuValue;
class ServerConfig;

class MenuConstraint {
public:
    virtual ~MenuConstraint() { }

    // NOTE: apply and canSetTo probably duplicate each other. Might be possible to write a generic
    // ApplyConstraints function that uses only canSetTo.
    
    virtual bool apply(MenuSelections &msel) = 0; // return true if something was changed.
    virtual bool canSetTo(const MenuSelections &msel, const string &id, int i) = 0;
         // ... returns true this constraint allows menu item "id" to be set to "i" (under the given menu selections).
};

// the following pops an argument list from the kfile:
shared_ptr<MenuConstraint> CreateMenuConstraint(const string &name, ServerConfig &sc);

#endif
